function N = nbm_chk(wght, alpha, beta)

%nbm_chk   Checks the parameters of a negative binomial mixture
%          and returns its dimension.
%          Use: N = nbm_chk(wght,alpha,beta).

% H2M/cnt Toolbox, $Revision: 2.2 $
% Olivier Capp, 2004/02/06 - $Date: 2004/03/15 14:02:47 $
% http://www.tsi.enst.fr/~cappe/h2m/

% Constants
% Max deviation from 1 (beware parameters may have been computed in float)
MAX_DEV = 1e-6;

% Check input arguments
error(nargchk(3, 3, nargin));

% Number of mixture components
N = length(wght);
if (any(wght < 0) | any(wght > 1))
  error('Mixture weights must be between 0 and 1.');
end
if (abs(sum(wght)-1) > MAX_DEV)
  error('Mixture weigths are not normalized.');
end
if (length(alpha) ~= N)
  error('Transition matrix and shape parameters must have the compatible sizes.');
end
if (any(alpha) <= 0)
  error('Shape parameters must be positive.');
end
if (length(beta) ~= N)
  error('Transition matrix and inverse scales must have the compatible sizes.');
end
if (any(beta) <= 0)
  error('Inverse scales must be positive.');
end
