%ex_fb_bms Script to check that the results of Forward-Backward and Backward
%          Markovian Smoothing are the same.

% H2M Toolbox, $Revision: 2.4 $
% Olivier Capp, 24/06/96 - 14/08/2001
% http://www.tsi.enst.fr/~cappe/h2m/

% The Backward Markovian Smoothing algorithms is described, for instance, as
% Algorithm 5.1.3 (p. 124) of
%    author =     {Olivier Capp and Eric Moulines and Tobias Rydn},
%    title =      {Inference in Hidden Markov Models},
%    publisher =  {Springer},
%    year =       {2005},
%    url =        {http://www.tsi.enst.fr/~cappe/ihmm/}
% and in Section V.A of
%    author =     {Y. Ephraim and N. Merhav},
%    title =      {Hidden {M}arkov processes},
%    journal =    {IEEE Trans. Inform. Theory},
%    year =       {2002},
%    volume =     {48},
%    pages =      {1518-1569}

clear all;

% Simulate some data (three states, diagonal covariances HMM)
d = 3;
mu = [0; 1; 2];
sigma2 = [0.5; 0.5; 0.5];
A = [0.8 0.1 0.1; 0.1 0.8 0.1; 0.1 0.1 0.8];
pi0 = [0.5 0.3 0.2];
T = 1000;
[X,s] = hmm_gen(A, pi0, mu, sigma2, T);

% Computation using the usual forward-backward algorithm
fprintf(1, 'Forward-Backward\n');
[alpha_b, beta_b, logl_b, dens] = hmm_fb(X, A, pi0, mu, sigma2);
[A_b,pi0_b] = hmm_tran(alpha_b,beta_b,dens,A,pi0);
gamma_b = alpha_b .* beta_b;
gamma_b = gamma_b ./ (sum(gamma_b')' * ones(1,d));

% Same thing with backward Markovian smoothing
fprintf(1, '\nBackward Markovian Smoothing\n');
[gamma, Ahat, logl] = hmm_post(X, A, pi0, mu, sigma2);

% Plot difference (these should be of the order of, at most, the numerical
% precision, that is, e-14)
fprintf(1, '\nLog-likelihood difference %g\n', logl-logl_b);
fprintf( 1, 'Posterior prob. difference %g\n', max(max(abs(gamma-gamma_b))) );
fprintf( 1, 'Transition matrix estim. difference %g\n', max(max(abs(Ahat-A_b))) );
