function [w, mu, Sigma, logl] = mix (X, w_ini, mu_ini, Sigma_ini, n_it, LOG_FLAG)

%mix       Performs multiple iterations of the EM algorithm for a mixture model.
%	   This is just a call to the lower-level functions.
%       Use: [w,mu,Sigma,logl] = mix (X,w_ini,mu_ini,Sigma_ini,
%       n_it,LOG_FLAG) where LOG_FLAG can be set to 1 to use logarithmic
%       computations (typically useful for very high dimensional, say p >
%       50, models). Default is to use linear scale computation.

% H2M Toolbox, $Revision: 2.3 $        
% Olivier Capp, 07/02/97 - $Date: 2004/11/03 14:16:49 $
% http://www.tsi.enst.fr/~cappe/h2m/

% Turn verbose mode off
QUIET = 1;
% Check input data and dimensions
error(nargchk(5, 6, nargin));
% Default to standard computation
if (nargin < 6)
  LOG_FLAG = 0;
end
% Check that the initialization value are correct
[N, p, DIAG_COV] = mix_chk(w_ini, mu_ini, Sigma_ini);
% Use supplied values as initialization
w = w_ini;
mu = mu_ini;
Sigma = Sigma_ini;

logl = zeros(1,n_it);
for i = 1:n_it
  if (LOG_FLAG == 1)
    [gamma, logl(i)] = mix_postl(X, w, mu, Sigma, QUIET);
  else
    [gamma, logl(i)] = mix_post(X, w, mu, Sigma, QUIET);
  end
  [mu, Sigma, w] = mix_par(X, gamma, DIAG_COV, QUIET);
  fprintf(1, 'Iteration %d:\t%.3f\n', i, logl(i));
end
